/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is OIOSAML Java Service Provider.
 *
 * The Initial Developer of the Original Code is Trifork A/S. Portions
 * created by Trifork A/S are Copyright (C) 2008 Danish National IT
 * and Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Joakim Recht <jre@trifork.com>
 *   Rolf Njor Jensen <rolf@trifork.com>
 *   Aage Nielsen <ani@openminds.dk>
 *
 */
package com.fr.plugin.adfs.saml;

import com.fr.base.TemplateUtils;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.PropertiesUtils;
import com.fr.locale.InterProviderFactory;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.stable.StringUtils;
import com.fr.stable.web.Device;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
/**
 * @Author fr.open
 * @Date 2021/7/21
 * @Description
 **/
@FunctionRecorder
public class SsoFilter extends AbstractGlobalRequestFilterProvider {

    private static final String[] notFilter = {
            "/decision/file",
            "/decision/resources",
            "/decision/login/config",
            "/decision/system",
            "/decision/login/slider",
            "/decision/remote",
            "/decision/login",
    };

    @Override
    public String filterName() {
        return "ynhg";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        String samlToken = req.getParameter("samlToken");
        try {
            if(isAccept(req) || isLogin(req)){
                if(req.getRequestURI().endsWith("hello.html")){
                    String root = TemplateUtils.render("${fineServletURL}");
                    res.sendRedirect(root);
                    return;
                }
                next(req, res, filterChain);
                return;
            }
            if(req.getRequestURI().endsWith("hello.html")){
                FineLoggerFactory.getLogger().info("start oa sso");
                SimpleDateFormat sdfDay = new SimpleDateFormat(
                        "yyyy-MM-dd");
                Date date = new Date();//系统当前日期
                String username = req.getParameter("loginid");;
                //解密
                try {
                    username = KeysUtil .decrypt(username, sdfDay.format(date));
                } catch (Exception e1) {
                    FineLoggerFactory.getLogger().error(e1.getMessage(),e1);
                }
                FineLoggerFactory.getLogger().info("get oa user is {}",username);
                if (StringUtils.isNotEmpty(username)) {
                    loginFromToken(req,res,username);
                    //next(req, res, filterChain);
                    String root = TemplateUtils.render("${fineServletURL}");
                    res.sendRedirect(root);
                    return;
                }
            }

            if (StringUtils.isNotBlank(samlToken)) {
                String key = PropertiesUtils.getProperties("conf").getProperty("key");
                FineLoggerFactory.getLogger().info("get key config is {}", key);
                String format = DesECBUtil.decryptDES(samlToken, key);
                FineLoggerFactory.getLogger().info("decode format is {}",format);
                String user = format.split("_")[0];
                String timeout = format.split("_")[1];
                String out = PropertiesUtils.getProperties("conf").getProperty("timeout");
                FineLoggerFactory.getLogger().info("get timeout config is {}", out);
                if(StringUtils.isBlank(out)){
                    out = "10";
                }
                if (Instant.ofEpochMilli(Long.valueOf(timeout)).plusSeconds(Long.valueOf(out)).isBefore(Instant.now())) {
                    setError(res, "format timeout!");
                    return;
                }
                if(!checkUser(user)){
                    setError(res, "user: 【"+user+"】 not exist !");
                    return;
                }
                loginFromToken(req,res,user);
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        FineLoggerFactory.getLogger().info("current request {} not login send redirect login",getUrl(req));
        String saml = PropertiesUtils.getProperties("conf").getProperty("saml");
        try {
            res.sendRedirect(saml);
        } catch (IOException e) {
            FineLoggerFactory.getLogger().error(e.getMessage(),e);
        }
    }

    private String getUrl(HttpServletRequest req) {
        String requestURL = req.getRequestURL().toString();
        String queryString = req.getQueryString();
        if(StringUtils.isNotBlank(queryString)){
            requestURL +="?"+queryString;
        }
        return requestURL;
    }

    private boolean isLogin(HttpServletRequest request) throws Exception {
        String oldToken = TokenResource.COOKIE.getToken(request);
        return oldToken != null && checkTokenValid(request, (String) oldToken);
    }

    private boolean checkTokenValid(HttpServletRequest req, String token) {
        try {
            Device device = NetworkHelper.getDevice(req);
            LoginService.getInstance().loginStatusValid(token, TerminalHandler.getTerminal(req, device));
            return true;
        } catch (Exception ignore) {
        }
        return false;
    }

    private boolean isAccept(HttpServletRequest req) {
        for (int i = 0; i < notFilter.length; i++) {
            if (req.getRequestURI().contains(notFilter[i])) {
                return true;
            }
        }
        if (req.getRequestURI().endsWith("/.css") || req.getRequestURI().endsWith("/.js")) {
            return true;
        }
        if(req.getRequestURI().endsWith("/view/form") || req.getRequestURI().endsWith("/view/report")){
            return StringUtils.isBlank(WebUtils.getHTTPRequestParameter(req, "viewlet"));
        }
        return false;
    }

    private void next(HttpServletRequest request, HttpServletResponse response, FilterChain chain) {
        try {
            chain.doFilter(request, response);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private boolean checkUser(String username) {
        User user = null;
        try {
            user = UserService.getInstance().getUserByUserName(username);
            FineLoggerFactory.getLogger().info("get user:" + user);
            if (user != null) {
                return true;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }


        return false;
    }

    private boolean loginFromToken(HttpServletRequest req, HttpServletResponse res, String username) throws Exception {
        try {
            if (StringUtils.isNotEmpty(username)) {
                FineLoggerFactory.getLogger().info("current username:" + username);
                String token = LoginService.getInstance().login(req, res, username);
                FineLoggerFactory.getLogger().info("get login token:" + token);
                req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, token);
                FineLoggerFactory.getLogger().info("username:" + username + "login success");
                return true;
            } else {
                FineLoggerFactory.getLogger().warn("username is null!");
                return false;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return false;
    }

    private void setError(HttpServletResponse res, String reason) {
        try {
            PrintWriter printWriter = WebUtils.createPrintWriter(res);
            Map<String, Object> map = new HashMap<>();
            map.put("result", InterProviderFactory.getProvider().getLocText("Fine-Engine_Error_Page_Result"));
            map.put("reason", reason);
            map.put("solution", InterProviderFactory.getProvider().getLocText("Fine-Engine_Please_Contact_Platform_Admin"));
            String page = WebServiceUtils.parseWebPageResourceSafe("com/fr/web/controller/decision/entrance/resources/unavailable.html", map);
            printWriter.write(page);
            printWriter.flush();
            printWriter.close();
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }
}

